package com.bpcl.lpg.core.cronjob;

import org.slf4j.LoggerFactory;
import com.bpcl.lpg.facades.ym.data.LpgYMV2ResponseData;
import java.util.Objects;
import org.apache.commons.lang3.StringUtils;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import com.bpcl.lpg.core.model.LpgContactUpdateModel;
import com.bpcl.lpg.core.model.LpgAutoPortabilityModel;
import com.bpcl.lpg.core.model.LpgDBCRequestModel;
import com.bpcl.lpg.core.model.LpgMechanicSRModel;
import com.bpcl.lpg.core.model.NewConnectionsModel;
import com.bpcl.lpg.core.model.CODRefillsModel;
import com.bpcl.lpg.core.model.LpgPaymentInfoModel;
import java.util.List;
import java.text.NumberFormat;
import com.bpcl.lpg.core.enums.GenericStatusEnum;
import com.bpcl.lpg.core.enums.RefillBookingStatusEnum;
import com.bpcl.lpg.core.enums.LpgPaymentStatusEnum;
import java.util.Calendar;
import com.bpcl.lpg.core.pojo.BusinessReportData;
import java.util.LinkedHashMap;
import java.util.Date;
import java.text.MessageFormat;
import com.bpcl.core.util.BPCLDateUtils;
import de.hybris.platform.cronjob.enums.CronJobStatus;
import de.hybris.platform.cronjob.enums.CronJobResult;
import de.hybris.platform.servicelayer.cronjob.PerformResult;
import com.bpcl.core.ym.notication.service.LpgV2NotificationEngineService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import com.bpcl.core.ym.notication.service.NotificationEngineService;
import com.bpcl.lpg.core.dao.ContactUpdateDao;
import com.bpcl.lpg.core.dao.AutoPortabilityReqDao;
import com.bpcl.lpg.core.dao.DbcRequestDao;
import com.bpcl.lpg.core.dao.MechanicServiceRequestDao;
import com.bpcl.lpg.core.dao.NewConnectionDao;
import com.bpcl.lpg.core.dao.CODRefillsReportDao;
import com.bpcl.lpg.core.payment.billdesk.dao.LPGPaymentInfoDao;
import org.slf4j.Logger;
import de.hybris.platform.cronjob.model.CronJobModel;
import de.hybris.platform.servicelayer.cronjob.AbstractJobPerformable;

public class LpgBusinessReportJob extends AbstractJobPerformable<CronJobModel>
{
    private static final Logger LOG;
    private static final long MILLIS_IN_A_DAY = 86400000L;
    private static final String DATE_PATTERN_WITH_TIME = "dd/MM/yyyy hh:mm:ss";
    private static final String DATE_PATTERN_WITHOUT_TIME = "dd/MM/yyyy";
    private static final String HEADER_MONTHLY_DATE_PATTERN = "MMM-yy";
    private static final String APP_START_DATE = "01/03/2021 00:00:00";
    private static final String SERV_REQ_START_DATE = "01/04/2021 00:00:00";
    private static final String CONTACT_UPDATE_REQ_START_DATE = "18/06/2021 00:00:00";
    private static final String START_TIME_STRING = " 00:00:00";
    private static final String END_TIME_STRING = " 23:59:59";
    private static final String PRINT_MAP_PATTERN = "{} -> {}";
    private static final String S_NO_1 = "1";
    private static final String S_NO_1_1 = "1.1";
    private static final String S_NO_1_2 = "1.2";
    private static final String S_NO_2 = "2";
    private static final String S_NO_2_1 = "2.1";
    private static final String S_NO_2_2 = "2.2";
    private static final String S_NO_3 = "3";
    private static final String S_NO_4 = "4";
    private static final String S_NO_5 = "5";
    private static final String S_NO_6 = "6";
    private static final String S_NO_7 = "7";
    private static final String S_NO_8 = "8";
    private static final String LABEL_TOTAL_SUCCESSFUL_BOOKING = "Total successful bookings";
    private static final String LABEL_SUCCESSFUL_ONLINE_BOOKING = "Successful online payment and successful bookings";
    private static final String LABEL_SUCCESSFUL_COD_BOOKING = "Successful COD bookings";
    private static final String LABEL_TOTAL_FAILED_BOOKING = "Total failed bookings";
    private static final String LABEL_FAILED_ONLINE_BOOKING = "Successful online payment but failed bookings";
    private static final String LABEL_FAILED_COD_BOOKING = "Failed COD bookings";
    private static final String LABEL_FAILED_BOOKING_PERCENTAGE = "Failed bookings percentage";
    private static final String LABEL_SUCCESSFUL_NEW_ENROLLMENTS = "Successful new enrolments";
    private static final String LABEL_SUCCESSFUL_MECHANIC_SR = "Successful mechanic service requests raised ";
    private static final String LABEL_SUCCESSFUL_DBC_REQUEST = "Successful double bottle connection requests raised ";
    private static final String LABEL_SUCCESSFUL_AUTO_PORTABILITY_REQ = "Successful distributor portability requests raised ";
    private static final String LABEL_SUCCESSFUL_CONTACT_UPDATE = "Successful mobile number change requests";
    private static final String REPORT_HEADER_SUPERSCRIPT_4 = "(4)";
    private static final String REPORT_HEADER_NUMBER = "S. No.";
    private static final String REPORT_HEADER_DAILY_COUNT = "Daily Count";
    private static final String REPORT_HEADER_DAILY_COUNT_DATE = "{dDate}";
    private static final String REPORT_HEADER_MONTHLY_COUNT = "Monthly Count";
    private static final String REPORT_HEADER_MONTHLY_COUNT_DATE = "{mDate}";
    private static final String REPORT_HEADER_CUMULATIVE_COUNT = "Cumulative Count";
    private static final String REPORT_CUMULATIVE_COUNT_START_DATE = "{cStart}";
    private static final String REPORT_CUMULATIVE_COUNT_END_DATE = "{cEnd}";
    private static final String REPORT_CONTACT_UPDATE_START_DATE = "{cuStart}";
    private static final String REPORT_CONTACT_UPDATE_END_DATE = "{cuEnd}";
    private static final String REPORT_HEADER_PARAM = "Parameter";
    private static final String REPORT_FOOTER_DAILY_START = "{dStart}";
    private static final String REPORT_FOOTER_DAILY_END = "{dEnd}";
    private static final String REPORT_FOOTER_MONTHLY_START = "{mStart}";
    private static final String REPORT_FOOTER_MONTHLY_END = "{mEnd}";
    private static final String NOTE_LINE_1 = "Kindly note:";
    private static final String NOTE_LINE_2 = "1. Figures stated above pertain to Bharatgas.";
    private static final String NOTE_LINE_3 = "2. The daily count is for the period {dStart} to {dEnd}.";
    private static final String NOTE_LINE_4 = "3. The monthly count is for the period {mStart} to {mEnd}.";
    private static final String NOTE_LINE_5 = "4. The cumulative count for parameters 5 to 7 are from {cStart} to {cEnd}.";
    private static final String NOTE_LINE_6 = "5. Figure in brackets indicates percentage of total for that parameter.";
    private static final String NOTE_LINE_7 = "6. Failed Bookings Percentage = Total Failed Bookings/(Total Successful Bookings + Total Failed Bookings) %";
    private static final String NOTE_LINE_8 = "7. The cumulative count for parameter 8 is from {cuStart} to {cuEnd}";
    private LPGPaymentInfoDao lpgPaymentInfoDao;
    private CODRefillsReportDao codRefillsReportDao;
    private NewConnectionDao newConnectionDao;
    private MechanicServiceRequestDao mechanicServiceRequestDao;
    private DbcRequestDao dbcRequestDao;
    private AutoPortabilityReqDao autoPortabilityReqDao;
    private ContactUpdateDao contactUpdateDao;
    private NotificationEngineService notificationEngineService;
    private ConfigurationService configurationService;
    private LpgV2NotificationEngineService lpgV2NotificationEngineService;
    
    public PerformResult perform(final CronJobModel cronjobModel) {
        try {
            final Date jobStartTime = cronjobModel.getStartTime();
            LpgBusinessReportJob.LOG.info("Cronjob processing started on {}", jobStartTime);
            final Date startTime = this.getDefaultStartDate();
            final Date endTime = this.getDefaultEndTime();
            final LinkedHashMap<String, BusinessReportData> statistics = this.prepareReportMap();
            final Date date;
            final Date monthEndTime;
            final Date cumulativeEndTime = monthEndTime = (date = (endTime.after(jobStartTime) ? jobStartTime : endTime));
            final Date dailyEndTime = date;
            final Date dailyStartTime = this.getStartOfDay();
            this.processDailyCount(statistics, dailyStartTime, dailyEndTime);
            final Date monthStartTime = this.getStartOfMonth(monthEndTime);
            this.processMonthlyCount(statistics, monthStartTime, monthEndTime);
            final Date cumulativeStartTime = this.getCumulativeStartTime();
            this.processCumulativeCount(statistics, cumulativeStartTime, cumulativeEndTime);
            final String emailSubjectLine = this.populateSubject(startTime, endTime);
            final StringBuilder emailBody = this.prepareEmailBody(statistics, dailyStartTime, dailyEndTime, monthStartTime, monthEndTime, cumulativeStartTime, cumulativeEndTime);
            if (this.sendEmail(emailSubjectLine, emailBody.toString())) {
                return new PerformResult(CronJobResult.SUCCESS, CronJobStatus.FINISHED);
            }
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.FINISHED);
        }
        catch (final Exception e) {
            LpgBusinessReportJob.LOG.error(e.getMessage(), e.getCause());
            final String body = MessageFormat.format("Business report cronjob is failed due to exception at {0}", BPCLDateUtils.getCurrentDateAsString("dd-MMM-yy h:mm a"));
            this.lpgV2NotificationEngineService.sendEmail("FAILED Business Report Job Alert", body, this.configurationService.getConfiguration().getString("lpg.alert.email.receiver.id"));
            return new PerformResult(CronJobResult.FAILURE, CronJobStatus.ABORTED);
        }
    }
    
    private StringBuilder prepareEmailBody(final LinkedHashMap<String, BusinessReportData> statistics, final Date dailyStart, final Date dailyEnd, final Date monthStart, final Date monthEnd, final Date cumulativeStart, final Date cumulativeEnd) {
        final StringBuilder emailBody = new StringBuilder("<html><head><style>table, th, td { border: 1px solid black; } table { width:75% } small { font-size: 10px } div {margin-left:.5in} </style></head><body>");
        emailBody.append(this.populateEmailBodyContent(statistics, BPCLDateUtils.dateToString(dailyEnd, "dd-MMM-yy"), BPCLDateUtils.dateToString(monthEnd, "MMM-yy"), BPCLDateUtils.dateToString(cumulativeStart, "dd-MMM-yy"), BPCLDateUtils.dateToString(cumulativeEnd, "dd-MMM-yy")));
        emailBody.append("<br/>");
        emailBody.append("<br/>");
        this.populateFooterNote(emailBody, dailyStart, dailyEnd, monthStart, monthEnd, cumulativeStart, cumulativeEnd);
        emailBody.append("</body></html>");
        return emailBody;
    }
    
    private void populateFooterNote(final StringBuilder emailBody, final Date dailyStart, final Date dailyEnd, final Date monthStart, final Date monthEnd, final Date cumulativeStartTime, final Date cumulativeEndTime) {
        final Date servReqStartDate = BPCLDateUtils.getDateAsSpecifiedFormat("01/04/2021 00:00:00", "dd/MM/yyyy hh:mm:ss");
        final Date filterDate = cumulativeStartTime.before(servReqStartDate) ? servReqStartDate : cumulativeStartTime;
        emailBody.append("<p>");
        final String noteString = "Kindly note:<br/>1. Figures stated above pertain to Bharatgas.<br/>2. The daily count is for the period {dStart} to {dEnd}.<br/>3. The monthly count is for the period {mStart} to {mEnd}.<br/>4. The cumulative count for parameters 5 to 7 are from {cStart} to {cEnd}.<br/>5. Figure in brackets indicates percentage of total for that parameter.<br/>6. Failed Bookings Percentage = Total Failed Bookings/(Total Successful Bookings + Total Failed Bookings) %<br/>7. The cumulative count for parameter 8 is from {cuStart} to {cuEnd}";
        final Date contactUpdateStartDate = BPCLDateUtils.getDateAsSpecifiedFormat("18/06/2021 00:00:00", "dd/MM/yyyy hh:mm:ss");
        emailBody.append(noteString.replace("{dStart}", BPCLDateUtils.dateToString(dailyStart, "dd-MMM-yy h:mm a")).replace("{dEnd}", BPCLDateUtils.dateToString(dailyEnd, "dd-MMM-yy h:mm a")).replace("{mStart}", BPCLDateUtils.dateToString(monthStart, "dd-MMM-yy h:mm a")).replace("{mEnd}", BPCLDateUtils.dateToString(monthEnd, "dd-MMM-yy h:mm a")).replace("{cStart}", BPCLDateUtils.dateToString(filterDate, "dd-MMM-yy h:mm a")).replace("{cEnd}", BPCLDateUtils.dateToString(cumulativeEndTime, "dd-MMM-yy h:mm a")).replace("{cuStart}", BPCLDateUtils.dateToString(contactUpdateStartDate, "dd-MMM-yy h:mm a")).replace("{cuEnd}", BPCLDateUtils.dateToString(cumulativeEndTime, "dd-MMM-yy h:mm a")));
        emailBody.append("</p>");
    }
    
    private Date getCumulativeStartTime() {
        return BPCLDateUtils.getDateAsSpecifiedFormat("01/03/2021 00:00:00", "dd/MM/yyyy hh:mm:ss");
    }
    
    private Date getStartOfDay() {
        return this.getDefaultStartDate();
    }
    
    private LinkedHashMap<String, BusinessReportData> prepareReportMap() {
        final LinkedHashMap<String, BusinessReportData> statistics = new LinkedHashMap<String, BusinessReportData>();
        statistics.put("1", new BusinessReportData("Total successful bookings"));
        statistics.put("1.1", new BusinessReportData(this.addMargin("Successful online payment and successful bookings")));
        statistics.put("1.2", new BusinessReportData(this.addMargin("Successful COD bookings")));
        statistics.put("2", new BusinessReportData("Total failed bookings"));
        statistics.put("2.1", new BusinessReportData(this.addMargin("Successful online payment but failed bookings")));
        statistics.put("2.2", new BusinessReportData(this.addMargin("Failed COD bookings")));
        statistics.put("3", new BusinessReportData("Failed bookings percentage"));
        statistics.put("4", new BusinessReportData("Successful new enrolments"));
        statistics.put("5", new BusinessReportData(this.appendSuperScript("Successful mechanic service requests raised ")));
        statistics.put("6", new BusinessReportData(this.appendSuperScript("Successful double bottle connection requests raised ")));
        statistics.put("7", new BusinessReportData(this.appendSuperScript("Successful distributor portability requests raised ")));
        statistics.put("8", new BusinessReportData("Successful mobile number change requests"));
        return statistics;
    }
    
    private String addMargin(final String label) {
        return "<div>" + label + "</div>";
    }
    
    private String appendSuperScript(final String label) {
        return label + "<sup>(4)</sup>";
    }
    
    private Date getStartOfMonth(final Date monthEnd) {
        final Calendar c = Calendar.getInstance();
        c.setTime(monthEnd);
        c.set(5, 1);
        return BPCLDateUtils.getDateAsSpecifiedFormat(BPCLDateUtils.dateToString(c.getTime(), "dd/MM/yyyy") + " 00:00:00", "dd/MM/yyyy hh:mm:ss");
    }
    
    private Date getDefaultStartDate() {
        final String yesterdayStart = this.getYesterdayDate() + " 00:00:00";
        return BPCLDateUtils.getDateAsSpecifiedFormat(yesterdayStart, "dd/MM/yyyy hh:mm:ss");
    }
    
    private Date getDefaultEndTime() {
        final String yesterdayEnd = this.getYesterdayDate() + " 23:59:59";
        return BPCLDateUtils.getDateAsSpecifiedFormat(yesterdayEnd, "dd/MM/yyyy hh:mm:ss");
    }
    
    private String getYesterdayDate() {
        final Date currentDate = new Date();
        final Date yesterday = new Date(currentDate.getTime() - 86400000L);
        return BPCLDateUtils.dateToString(yesterday, "dd/MM/yyyy");
    }
    
    private void processDailyCount(final LinkedHashMap<String, BusinessReportData> statistics, final Date startTime, final Date endTime) {
        LpgBusinessReportJob.LOG.info("Start date for daily dataset: {}", startTime);
        LpgBusinessReportJob.LOG.info("End date for daily dataset: {}", endTime);
        final List<LpgPaymentInfoModel> successfulBookingList = this.lpgPaymentInfoDao.getOnlineTransactions(startTime, endTime, LpgPaymentStatusEnum.SUCCESSFUL, RefillBookingStatusEnum.SUCCESSFUL);
        final List<LpgPaymentInfoModel> failedRefillBookingList = this.lpgPaymentInfoDao.getOnlineTransactions(startTime, endTime, LpgPaymentStatusEnum.SUCCESSFUL, RefillBookingStatusEnum.FAILED);
        final List<CODRefillsModel> successfulCODRefillList = this.codRefillsReportDao.getCODRefillTransaction(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<CODRefillsModel> failedCODRefillList = this.codRefillsReportDao.getCODRefillTransaction(startTime, endTime, GenericStatusEnum.FAILED);
        final List<NewConnectionsModel> successfulNewConnectionsList = this.newConnectionDao.getNewConnectionsTransaction(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgMechanicSRModel> successfulMechanicReqList = this.mechanicServiceRequestDao.getMechanicServiceRequestsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgDBCRequestModel> successfulDbcReqList = this.dbcRequestDao.getDbcRequestsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgAutoPortabilityModel> successfulPortabilityList = this.autoPortabilityReqDao.getAutoPortabilityReqsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgContactUpdateModel> successfulContactUpdateList = this.contactUpdateDao.getContactUpdateReqsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final NumberFormat nf = NumberFormat.getInstance();
        final int successfulOnlineCount = successfulBookingList.size();
        final int successfulCodCount = successfulCODRefillList.size();
        final int totalSuccessful = successfulOnlineCount + successfulCodCount;
        statistics.get("1").setDailyCount(nf.format(totalSuccessful));
        statistics.get("1.1").setDailyCount(this.populateCountWithPercentage(nf, totalSuccessful, successfulOnlineCount));
        statistics.get("1.2").setDailyCount(this.populateCountWithPercentage(nf, totalSuccessful, successfulCodCount));
        final int failedOnlineCount = failedRefillBookingList.size();
        final int failedCodCount = failedCODRefillList.size();
        final int totalFailed = failedOnlineCount + failedCodCount;
        statistics.get("2").setDailyCount(nf.format(totalFailed));
        statistics.get("2.1").setDailyCount(this.populateCountWithPercentage(nf, totalFailed, failedOnlineCount));
        statistics.get("2.2").setDailyCount(this.populateCountWithPercentage(nf, totalFailed, failedCodCount));
        statistics.get("3").setDailyCount(this.populatePercentage(totalSuccessful + totalFailed, totalFailed));
        statistics.get("4").setDailyCount(nf.format(successfulNewConnectionsList.size()));
        statistics.get("5").setDailyCount(nf.format(successfulMechanicReqList.size()));
        statistics.get("6").setDailyCount(nf.format(successfulDbcReqList.size()));
        statistics.get("7").setDailyCount(nf.format(successfulPortabilityList.size()));
        statistics.get("8").setDailyCount(nf.format(successfulContactUpdateList.size()));
        statistics.forEach((s, v) -> LpgBusinessReportJob.LOG.info("{} -> {}", v.getLabel(), v.getDailyCount()));
    }
    
    private String populatePercentage(final double total, final double part) {
        final DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(0);
        df.setRoundingMode(RoundingMode.HALF_UP);
        final double percentage = (total > 0.0) ? (part / total * 100.0) : 0.0;
        return df.format(percentage);
    }
    
    private String populateCountWithPercentage(final NumberFormat nf, final double total, final double part) {
        final StringBuilder sb = new StringBuilder(nf.format(part));
        sb.append(" ");
        final DecimalFormat df = new DecimalFormat();
        df.setMaximumFractionDigits(0);
        df.setRoundingMode(RoundingMode.HALF_UP);
        final double percentage = (total > 0.0) ? (part / total * 100.0) : 0.0;
        sb.append("(").append(df.format(percentage)).append("%").append(")");
        return sb.toString();
    }
    
    private void processMonthlyCount(final LinkedHashMap<String, BusinessReportData> statistics, final Date startTime, final Date endTime) {
        LpgBusinessReportJob.LOG.info("Start date for monthly dataset: {}", startTime);
        LpgBusinessReportJob.LOG.info("End date for monthly dataset: {}", endTime);
        final List<LpgPaymentInfoModel> successfulBookingList = this.lpgPaymentInfoDao.getOnlineTransactions(startTime, endTime, LpgPaymentStatusEnum.SUCCESSFUL, RefillBookingStatusEnum.SUCCESSFUL);
        final List<LpgPaymentInfoModel> failedRefillBookingList = this.lpgPaymentInfoDao.getOnlineTransactions(startTime, endTime, LpgPaymentStatusEnum.SUCCESSFUL, RefillBookingStatusEnum.FAILED);
        final List<CODRefillsModel> successfulCODRefillList = this.codRefillsReportDao.getCODRefillTransaction(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<CODRefillsModel> failedCODRefillList = this.codRefillsReportDao.getCODRefillTransaction(startTime, endTime, GenericStatusEnum.FAILED);
        final List<NewConnectionsModel> successfulNewConnectionsList = this.newConnectionDao.getNewConnectionsTransaction(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgMechanicSRModel> successfulMechanicReqList = this.mechanicServiceRequestDao.getMechanicServiceRequestsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgDBCRequestModel> successfulDbcReqList = this.dbcRequestDao.getDbcRequestsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgAutoPortabilityModel> successfulPortabilityList = this.autoPortabilityReqDao.getAutoPortabilityReqsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgContactUpdateModel> successfulContactUpdateList = this.contactUpdateDao.getContactUpdateReqsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final NumberFormat nf = NumberFormat.getInstance();
        final int successfulOnlineCount = successfulBookingList.size();
        final int successfulCodCount = successfulCODRefillList.size();
        final int totalSuccessful = successfulOnlineCount + successfulCodCount;
        statistics.get("1").setMonthlyCount(nf.format(totalSuccessful));
        statistics.get("1.1").setMonthlyCount(this.populateCountWithPercentage(nf, totalSuccessful, successfulOnlineCount));
        statistics.get("1.2").setMonthlyCount(this.populateCountWithPercentage(nf, totalSuccessful, successfulCodCount));
        final int failedOnlineCount = failedRefillBookingList.size();
        final int failedCodCount = failedCODRefillList.size();
        final int totalFailed = failedOnlineCount + failedCodCount;
        statistics.get("2").setMonthlyCount(nf.format(totalFailed));
        statistics.get("2.1").setMonthlyCount(this.populateCountWithPercentage(nf, totalFailed, failedOnlineCount));
        statistics.get("2.2").setMonthlyCount(this.populateCountWithPercentage(nf, totalFailed, failedCodCount));
        statistics.get("3").setMonthlyCount(this.populatePercentage(totalSuccessful + totalFailed, totalFailed));
        statistics.get("4").setMonthlyCount(nf.format(successfulNewConnectionsList.size()));
        statistics.get("5").setMonthlyCount(nf.format(successfulMechanicReqList.size()));
        statistics.get("6").setMonthlyCount(nf.format(successfulDbcReqList.size()));
        statistics.get("7").setMonthlyCount(nf.format(successfulPortabilityList.size()));
        statistics.get("8").setMonthlyCount(nf.format(successfulContactUpdateList.size()));
        statistics.forEach((s, v) -> LpgBusinessReportJob.LOG.info("{} -> {}", v.getLabel(), v.getMonthlyCount()));
    }
    
    private void processCumulativeCount(final LinkedHashMap<String, BusinessReportData> statistics, Date startTime, final Date endTime) {
        LpgBusinessReportJob.LOG.info("Start date for cumulative dataset: {}", startTime);
        LpgBusinessReportJob.LOG.info("End date for cumulative dataset: {}", endTime);
        final List<LpgPaymentInfoModel> successfulBookingList = this.lpgPaymentInfoDao.getOnlineTransactions(startTime, endTime, LpgPaymentStatusEnum.SUCCESSFUL, RefillBookingStatusEnum.SUCCESSFUL);
        final List<LpgPaymentInfoModel> failedRefillBookingList = this.lpgPaymentInfoDao.getOnlineTransactions(startTime, endTime, LpgPaymentStatusEnum.SUCCESSFUL, RefillBookingStatusEnum.FAILED);
        final List<CODRefillsModel> successfulCODRefillList = this.codRefillsReportDao.getCODRefillTransaction(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<CODRefillsModel> failedCODRefillList = this.codRefillsReportDao.getCODRefillTransaction(startTime, endTime, GenericStatusEnum.FAILED);
        final List<NewConnectionsModel> successfulNewConnectionsList = this.newConnectionDao.getNewConnectionsTransaction(startTime, endTime, GenericStatusEnum.SUCCESS);
        startTime = BPCLDateUtils.getDateAsSpecifiedFormat("01/04/2021 00:00:00", "dd/MM/yyyy hh:mm:ss");
        final List<LpgMechanicSRModel> successfulMechanicReqList = this.mechanicServiceRequestDao.getMechanicServiceRequestsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgDBCRequestModel> successfulDbcReqList = this.dbcRequestDao.getDbcRequestsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final List<LpgAutoPortabilityModel> successfulPortabilityList = this.autoPortabilityReqDao.getAutoPortabilityReqsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        startTime = BPCLDateUtils.getDateAsSpecifiedFormat("18/06/2021 00:00:00", "dd/MM/yyyy hh:mm:ss");
        final List<LpgContactUpdateModel> successfulContactUpdateList = this.contactUpdateDao.getContactUpdateReqsByStatus(startTime, endTime, GenericStatusEnum.SUCCESS);
        final NumberFormat nf = NumberFormat.getInstance();
        final int successfulOnlineCount = successfulBookingList.size();
        final int successfulCodCount = successfulCODRefillList.size();
        final int totalSuccessful = successfulOnlineCount + successfulCodCount;
        statistics.get("1").setCumulativeCount(nf.format(totalSuccessful));
        statistics.get("1.1").setCumulativeCount(this.populateCountWithPercentage(nf, totalSuccessful, successfulOnlineCount));
        statistics.get("1.2").setCumulativeCount(this.populateCountWithPercentage(nf, totalSuccessful, successfulCodCount));
        final int failedOnlineCount = failedRefillBookingList.size();
        final int failedCodCount = failedCODRefillList.size();
        final int totalFailed = failedOnlineCount + failedCodCount;
        statistics.get("2").setCumulativeCount(nf.format(totalFailed));
        statistics.get("2.1").setCumulativeCount(this.populateCountWithPercentage(nf, totalFailed, failedOnlineCount));
        statistics.get("2.2").setCumulativeCount(this.populateCountWithPercentage(nf, totalFailed, failedCodCount));
        statistics.get("3").setCumulativeCount(this.populatePercentage(totalSuccessful + totalFailed, totalFailed));
        statistics.get("4").setCumulativeCount(nf.format(successfulNewConnectionsList.size()));
        statistics.get("5").setCumulativeCount(nf.format(successfulMechanicReqList.size()));
        statistics.get("6").setCumulativeCount(nf.format(successfulDbcReqList.size()));
        statistics.get("7").setCumulativeCount(nf.format(successfulPortabilityList.size()));
        statistics.get("8").setCumulativeCount(nf.format(successfulContactUpdateList.size()));
        statistics.forEach((s, v) -> LpgBusinessReportJob.LOG.info("{} -> {}", v.getLabel(), v.getCumulativeCount()));
    }
    
    private boolean sendEmail(final String emailSubjectLine, final String emailBody) {
        final String userId = this.configurationService.getConfiguration().getString("lpg.business.report.receiver.id");
        if (StringUtils.isBlank((CharSequence)userId)) {
            LpgBusinessReportJob.LOG.error("No recipients mentioned, Hence email couldn't be triggered");
            return Boolean.FALSE;
        }
        final LpgYMV2ResponseData lpgYMV2ResponseData = this.lpgV2NotificationEngineService.sendEmail(emailSubjectLine, emailBody, userId);
        if (Objects.isNull(lpgYMV2ResponseData) || StringUtils.isBlank((CharSequence)lpgYMV2ResponseData.getMsgId())) {
            LpgBusinessReportJob.LOG.error("Email wasn't sent");
            return Boolean.FALSE;
        }
        LpgBusinessReportJob.LOG.info("Email sent successfully");
        return Boolean.TRUE;
    }
    
    private String populateSubject(final Date startTime, final Date endTime) {
        final StringBuilder sub = new StringBuilder(this.configurationService.getConfiguration().getString("lpg.business.report.email.subject"));
        final String startDateString = BPCLDateUtils.dateToString(startTime, "dd-MMM-yyyy");
        final String endDateString = BPCLDateUtils.dateToString(endTime, "dd-MMM-yyyy");
        if (StringUtils.equalsIgnoreCase((CharSequence)startDateString, (CharSequence)endDateString)) {
            sub.append(" for ").append(startDateString);
        }
        else {
            sub.append(" from ").append(BPCLDateUtils.dateToString(startTime, "dd-MMM-yyyy h:mm a")).append(" to ").append(BPCLDateUtils.dateToString(endTime, "dd-MMM-yyyy h:mm a"));
        }
        return sub.toString();
    }
    
    private String populateEmailBodyContent(final LinkedHashMap<String, BusinessReportData> statistics, final String dailyCountDate, final String monthlyCountDate, final String cStartDate, final String cEndDate) {
        final StringBuilder body = new StringBuilder();
        body.append("<table>").append("<tr>").append("<th>").append("S. No.").append("</th>").append("<th>").append("Parameter").append("</th>").append("<th>").append("Daily Count").append("<br/>").append("{dDate}").append("</th>").append("<th>").append("Monthly Count").append("<br/>").append("{mDate}").append("</th>").append("<th>").append("Cumulative Count").append("<br/>").append("<small>").append("[").append("{cStart}").append(" - ").append("{cEnd}").append("]").append("</small>").append("</th>").append("</tr>");
        statistics.forEach((s, v) -> body.append("<tr>").append("<td style='text-align: center'>").append(s).append("</td>").append("<td>").append(v.getLabel()).append("</td>").append("<td style='text-align: right'>").append(v.getDailyCount()).append("</td>").append("<td style='text-align: right'>").append(v.getMonthlyCount()).append("</td>").append("<td style='text-align: right'>").append(v.getCumulativeCount()).append("</td>").append("</tr>"));
        body.append("</table>");
        return body.toString().replace("{dDate}", dailyCountDate).replace("{mDate}", monthlyCountDate).replace("{cStart}", cStartDate).replace("{cEnd}", cEndDate);
    }
    
    public boolean isAbortable() {
        return Boolean.TRUE;
    }
    
    public void setLpgPaymentInfoDao(final LPGPaymentInfoDao lpgPaymentInfoDao) {
        this.lpgPaymentInfoDao = lpgPaymentInfoDao;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public void setCodRefillsReportDao(final CODRefillsReportDao codRefillsReportDao) {
        this.codRefillsReportDao = codRefillsReportDao;
    }
    
    public void setNewConnectionDao(final NewConnectionDao newConnectionDao) {
        this.newConnectionDao = newConnectionDao;
    }
    
    public void setMechanicServiceRequestDao(final MechanicServiceRequestDao mechanicServiceRequestDao) {
        this.mechanicServiceRequestDao = mechanicServiceRequestDao;
    }
    
    public void setDbcRequestDao(final DbcRequestDao dbcRequestDao) {
        this.dbcRequestDao = dbcRequestDao;
    }
    
    public void setAutoPortabilityReqDao(final AutoPortabilityReqDao autoPortabilityReqDao) {
        this.autoPortabilityReqDao = autoPortabilityReqDao;
    }
    
    public void setContactUpdateDao(final ContactUpdateDao contactUpdateDao) {
        this.contactUpdateDao = contactUpdateDao;
    }
    
    public void setNotificationEngineService(final NotificationEngineService notificationEngineService) {
        this.notificationEngineService = notificationEngineService;
    }
    
    public LpgV2NotificationEngineService getLpgV2NotificationEngineService() {
        return this.lpgV2NotificationEngineService;
    }
    
    public void setLpgV2NotificationEngineService(final LpgV2NotificationEngineService lpgV2NotificationEngineService) {
        this.lpgV2NotificationEngineService = lpgV2NotificationEngineService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgBusinessReportJob.class);
    }
}
